/**
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * a) This file is available under the Apache License, Version 2.0.
 * b) When publishing any derivative work or results obtained using this software, you agree to cite the following paper:
 *    Tomas Koutny and Martin Ubl, "SmartCGMS as a Testbed for a Blood-Glucose Level Prediction and/or 
 *    Control Challenge with (an FDA-Accepted) Diabetic Patient Simulation", Procedia Computer Science,  
 *    Volume 177, pp. 354-362, 2020
 */

#include "descriptor.h"
#include "generated_model.h"

#include <scgms/iface/UIIface.h>
#include <scgms/lang/dstrings.h>
#include <scgms/rtl/manufactory.h>
#include <scgms/rtl/hresult.h>
#include <scgms/utils/descriptor_utils.h>

#include <vector>

namespace generated123_model {

	// BP NOTE: tohle muzete pojmenovat podle toho, jak si uzivatel parametry nazve
	const wchar_t* model_param_ui_names[model_param_count] = {
		L"p1",
		L"p2",
		L"p3",
		L"p4",
		L"p5",
	};

	// BP NOTE: tohle jen nagenerujte na pocet - vsechny parametry modelu budou double
	const scgms::NModel_Parameter_Value model_param_types[model_param_count] = {
		scgms::NModel_Parameter_Value::mptDouble,
		scgms::NModel_Parameter_Value::mptDouble,
		scgms::NModel_Parameter_Value::mptDouble,
		scgms::NModel_Parameter_Value::mptDouble,
		scgms::NModel_Parameter_Value::mptDouble
	};

	// BP NOTE: tohle je pocet vystupnich signalu, generujte dle konfigurace
	constexpr size_t number_of_calculated_signals = 1;

	// BP NOTE: muzete pojmenovat podle toho, co si uzivatel zvoli
	const wchar_t* signal_Output_1_Name = L"Output signal 1";

	// BP NOTE: pro kazdy signal vygenerujte deskriptor dle sablony
	const scgms::TSignal_Descriptor signal_Output_1_desc{
		// BP NOTE: generovat
		signal_Output_1,
		signal_Output_1_Name,

		// BP NOTE: ponechat
		dsmmol_per_L,
		scgms::NSignal_Unit::mmol_per_L,

		// BP NOTE: mozno generovat
		0xFFFF0000, // BP NOTE: muzete vygenerovat nejakou nahodnou barvu
		0xFFFF0000, // BP NOTE: sem pouzijte tu samou

		// BP NOTE: ponechat
		scgms::NSignal_Visualization::smooth,
		scgms::NSignal_Mark::none,
		nullptr,
		1.0
	};

	// BP NOTE: sem jen naklopte vygenerovane GUID vystupnich signalu
	const GUID calculated_signal_ids[number_of_calculated_signals] = {
		signal_Output_1
	};

	// BP NOTE: sem jen naklopte vygenerovane nazvy vystupnich signalu
	const wchar_t* calculated_signal_names[number_of_calculated_signals] = {
		signal_Output_1_Name
	};

	// BP NOTE: tohle vygenerujte jen podle poctu vystupnich signalu, vsechny budou stejne (scgms::signal_Synchronization)
	const GUID reference_signal_ids[number_of_calculated_signals] = {
		scgms::signal_Synchronization
	};

	// BP NOTE: tohle muzete pojmenovat podle toho, jak uzivatel pojmenuje svuj model
	const wchar_t* model_name = L"Generated Model 123";

	// BP NOTE: tohle nechte, jak je
	scgms::TModel_Descriptor desc = {
		model_id,
		scgms::NModel_Flags::Discrete_Model,
		model_name,
		nullptr,
		model_param_count,
		1,
		model_param_types,
		model_param_ui_names,
		nullptr,
		lower_bounds.vector,
		default_parameters.vector,
		upper_bounds.vector,

		number_of_calculated_signals,
		calculated_signal_ids,
		reference_signal_ids,
	};
}

//// BP NOTE: NOVINKA: tohle pridejte:

std::vector<scgms::TSignal_Descriptor> signal_descriptors{
	//// BP NOTE: NOVINKA: tady generujte jen seznam signalu
	generated123_model::signal_Output_1_desc
};

//// BP NOTE: NOVINKA: tohle pridejte a nechte jak je
DLL_EXPORT HRESULT IfaceCalling do_get_signal_descriptors(scgms::TSignal_Descriptor** begin, scgms::TSignal_Descriptor** end) {

	if (signal_descriptors.empty()) {
		return S_FALSE;
	}

	*begin = &signal_descriptors[0];
	*end = &signal_descriptors[signal_descriptors.size()-1] + 1;
	return S_OK;
}
//// BP NOTE: NOVINKA: az sem

// BP NOTE: nechte, jak je, jen generujte spravne namespace
DLL_EXPORT HRESULT IfaceCalling do_get_model_descriptors(scgms::TModel_Descriptor **begin, scgms::TModel_Descriptor**end) {
	*begin = &generated123_model::desc; // BP NOTE: tady
	*end = *begin + 1;
	return S_OK;
}

// BP NOTE: nechte, jak je, jen generujte spravne namespace a jmeno tridy
DLL_EXPORT HRESULT IfaceCalling do_create_discrete_model(const GUID* model_id, scgms::IModel_Parameter_Vector* parameters, scgms::IFilter* output, scgms::IDiscrete_Model** model) {

	if (!model_id || !parameters || !output || !model) {
		return E_INVALIDARG;
	}

	if (*model_id == generated123_model::model_id) { // BP NOTE: tady a nize
		return Manufacture_Object<CGenerated123_Model>(model, parameters, output);
	}

	return E_NOTIMPL;
}
